package com.netease.nis.captcha_plugin_flutter

import android.app.Activity
import android.content.Context
import android.text.TextUtils
import android.util.Log
import com.netease.nis.captcha.Captcha
import com.netease.nis.captcha.CaptchaConfiguration
import com.netease.nis.captcha.CaptchaConfiguration.LangType
import com.netease.nis.captcha.CaptchaListener
import io.flutter.plugin.common.EventChannel
import io.flutter.plugin.common.MethodCall
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.MainScope
import kotlinx.coroutines.launch

/**
 * Created by hzhuqi on 2020/9/21
 */
class CaptchaHelper : CoroutineScope by MainScope() {
    var events: EventChannel.EventSink? = null

    fun init(activity: Activity, call: MethodCall) {
        val captchaId = call.argument<String>("captcha_id")
        val isDebug = call.argument<Boolean>("is_debug") ?: false //是否开启debug模式
        val isNoSenseMode = call.argument<Boolean>("is_no_sense_mode") ?: false//是否智能无感知
        val dimAmount = call.argument<Float>("dimAmount") ?: 0.5f //弹窗透明度
        val controlBarStartUrl = call.argument<String>("control_bar_start_url") ?: "" //自定义滑块开始背景
        val controlBarMovingUrl = call.argument<String>("control_bar_moving_url") ?: ""//自定义滑块滑动背景
        val controlBarErrorUrl = call.argument<String>("control_bar_error_url") ?: "" //自定义滑块错误背景
        val isTouchOutsideDisappear =
            call.argument<Boolean>("is_touch_outside_disappear") ?: true //点击弹窗外部是否可以关闭验证码
        val timeout = call.argument<Long>("timeout") ?: 1000 * 10 //超时时间
        val isHideCloseBtn = call.argument<Boolean>("is_hide_close_button") ?: false //是否隐藏关闭按钮
        val useDefaultFallback = call.argument<Boolean>("use_default_fallback") ?: true //是否采用默认降级
        val failedMaxRetryCount = call.argument<Int>("failed_max_retry_count") ?: 3 //失败后尝试最大次数
        val languageType = call.argument<String>("language_type") ?: "" //多语言语言类型

        if (captchaId != null) {
            val builder = CaptchaConfiguration.Builder()
            builder.captchaId(captchaId)
            builder.listener(captchaListener)
            builder.debug(isDebug)
            if (isNoSenseMode) {
                builder.mode(CaptchaConfiguration.ModeType.MODE_INTELLIGENT_NO_SENSE)
            }
            if (timeout != 0L) {
                builder.timeout(timeout)
            }
            if (dimAmount != 0f) {
                builder.backgroundDimAmount(dimAmount)
            }
            if (failedMaxRetryCount != 0) {
                builder.failedMaxRetryCount(failedMaxRetryCount)
            }
            if (!TextUtils.isEmpty(controlBarMovingUrl)) {
                builder.controlBarImageUrl(
                    controlBarStartUrl,
                    controlBarMovingUrl,
                    controlBarErrorUrl
                )
            }
            builder.touchOutsideDisappear(isTouchOutsideDisappear)
            builder.useDefaultFallback(useDefaultFallback)
            builder.hideCloseButton(isHideCloseBtn)
            if (!TextUtils.isEmpty(languageType)) {
                builder.languageType(string2LangType(languageType))
            }
            Captcha.getInstance().init(builder.build(activity))
        } else {
            Log.i("CaptchaHelper", "业务id必须传")
        }
    }

    fun showCaptcha() {
        Captcha.getInstance().validate()
    }

    fun destroy() {
        Captcha.getInstance().destroy()
    }

    private var captchaListener: CaptchaListener = object : CaptchaListener {
        override fun onReady() {
        }

        override fun onValidate(result: String?, validate: String?, msg: String?) {
            val data = HashMap<String, Any?>()
            data["validate"] = validate
            data["result"] = result
            data["msg"] = msg
            launch(Dispatchers.Main) {
                events?.success(data)
            }
        }

        override fun onError(code: Int, msg: String?) {
            val data = HashMap<String, Any?>()
            data["code"] = code
            data["msg"] = msg
            launch(Dispatchers.Main) {
                events?.success(data)
            }
        }

        override fun onClose(closeType: Captcha.CloseType?) {
        }
    }

    private fun string2LangType(langTypeStr: String): LangType {
        return when (langTypeStr) {
            "zh-TW" -> {
                LangType.LANG_ZH_TW
            }
            "en" -> {
                LangType.LANG_EN
            }
            "ja" -> {
                LangType.LANG_JA
            }
            "ko" -> {
                LangType.LANG_KO
            }
            "th" -> {
                LangType.LANG_TH
            }
            "vi" -> {
                LangType.LANG_VI
            }
            "fr" -> {
                LangType.LANG_FR
            }
            "ru" -> {
                LangType.LANG_RU
            }
            "ar" -> {
                LangType.LANG_AR
            }
            "de" -> {
                LangType.LANG_DE
            }
            "it" -> {
                LangType.LANG_IT
            }
            "he" -> {
                LangType.LANG_HE
            }
            "hi" -> {
                LangType.LANG_HI
            }
            "id" -> {
                LangType.LANG_ID
            }
            "my" -> {
                LangType.LANG_MY
            }
            "lo" -> {
                LangType.LANG_LO
            }
            "ms" -> {
                LangType.LANG_MS
            }
            "pl" -> {
                LangType.LANG_PL
            }
            "pt" -> {
                LangType.LANG_PT
            }
            "es" -> {
                LangType.LANG_ES
            }
            "tr" -> {
                LangType.LANG_TR
            }
            else -> LangType.LANG_ZH_CN
        }
    }
}