#import "CaptchaPluginFlutterPlugin.h"
#import <VerifyCode/NTESVerifyCodeManager.h>

@interface CaptchaPluginFlutterPlugin() <NTESVerifyCodeManagerDelegate>

@property(nonatomic, strong) NTESVerifyCodeManager *manager;

@end

@implementation CaptchaPluginFlutterPlugin

+ (CaptchaPluginFlutterPlugin *)sharedInstance {
    static dispatch_once_t onceToken = 0;
    static CaptchaPluginFlutterPlugin *sharedObject = nil;
    dispatch_once(&onceToken, ^{
        sharedObject = [[CaptchaPluginFlutterPlugin alloc] init];
    });

    return sharedObject;
}


+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
    FlutterMethodChannel* channel = [FlutterMethodChannel methodChannelWithName:@"yd_captcha_flutter_method_channel"
                                                                binaryMessenger:[registrar messenger]];

    CaptchaPluginFlutterPlugin* instance = [[CaptchaPluginFlutterPlugin alloc] init];
    instance.channel = channel;
    [registrar addMethodCallDelegate:instance channel:channel];

     FlutterViewController *view = (FlutterViewController *)[UIApplication sharedApplication].delegate.window.rootViewController;
     FlutterEventChannel* eventChannel= [FlutterEventChannel eventChannelWithName:@"yd_captcha_flutter_event_channel" binaryMessenger:view.binaryMessenger];
     [eventChannel setStreamHandler:[CaptchaPluginFlutterPlugin sharedInstance]];
}

- (FlutterError* _Nullable)onListenWithArguments:(id _Nullable)arguments
                                      eventSink:(FlutterEventSink)events {
      if (events) {
          [CaptchaPluginFlutterPlugin sharedInstance].eventSink = events;
      }

    return nil;
}

- (FlutterError * _Nullable)onCancelWithArguments:(id _Nullable)arguments {
    return nil;
}


- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
    NSString *methodName = call.method;
    if ([methodName isEqualToString:@"init"]) {
        [self init:call result:result];
    } else if ([methodName isEqualToString:@"showCaptcha"]) {
        [self showCaptcha:call result:result];
    } else if ([methodName isEqualToString:@"destroy"]) {
        [self destroyCaptcha:call result:result];
    } else {
        result(FlutterMethodNotImplemented);
    }
}

- (void)init:(FlutterMethodCall*) call result:(FlutterResult)resultDict {

    NSDictionary *options = call.arguments;
    self.manager = [NTESVerifyCodeManager getInstance];
    self.manager.delegate = self;

   NSString *captchaid;
   if ([options isKindOfClass:[NSDictionary class]]) {
       captchaid = [options objectForKey:@"captcha_id"];
       BOOL is_no_sense_mode = [[options objectForKey:@"is_no_sense_mode"] boolValue];
       BOOL is_hide_close_button = [[options objectForKey:@"is_hide_close_button"] boolValue];
       BOOL is_touch_outside_disappear = [[options objectForKey:@"is_touch_outside_disappear"] boolValue];
       BOOL use_default_fallback = [[options objectForKey:@"use_default_fallback"] boolValue];
       int failed_max_retry_count = [[options objectForKey:@"failed_max_retry_count"] intValue];
       int timeout = [[options objectForKey:@"timeout"] intValue];
       if (timeout > 0) {
           timeout = timeout / 1000;
       } else {
           timeout = 7;
       }
       NSString *language_type = [options objectForKey:@"language_type"];
       BOOL is_debug = [options objectForKey:@"is_debug"];
       [self.manager enableLog:is_debug];
       float dimAmount = [[options objectForKey:@"dimAmount"] floatValue];
       if (is_no_sense_mode) {
           self.manager.mode = NTESVerifyCodeBind;
       } else {
           self.manager.mode = NTESVerifyCodeNormal;
       }
       if (failed_max_retry_count > 0) {
           self.manager.fallBackCount = failed_max_retry_count;
       }
       self.manager.openFallBack = use_default_fallback;
       self.manager.closeButtonHidden = is_hide_close_button;
       self.manager.shouldCloseByTouchBackground = is_touch_outside_disappear;
       [self.manager configureVerifyCode:captchaid timeout:timeout];
       if (dimAmount != 0.0) {
           self.manager.alpha = dimAmount;
       }

       if ([language_type isKindOfClass:[NSString class]]) {
           if ([language_type isEqualToString:@"zh-TW"]) {
               self.manager.lang = NTESVerifyCodeLangTW;
           } else if ([language_type isEqualToString:@"en"]) {
               self.manager.lang = NTESVerifyCodeLangEN;
           } else if ([language_type isEqualToString:@"ja"]) {
               self.manager.lang = NTESVerifyCodeLangJP;
           } else if ([language_type isEqualToString:@"ko"]) {
               self.manager.lang = NTESVerifyCodeLangKR;
           } else if ([language_type isEqualToString:@"th"]) {
               self.manager.lang = NTESVerifyCodeLangTL;
           } else if ([language_type isEqualToString:@"vi"]) {
               self.manager.lang = NTESVerifyCodeLangVT;
           } else if ([language_type isEqualToString:@"fr"]) {
               self.manager.lang = NTESVerifyCodeLangFRA;
           } else if ([language_type isEqualToString:@"ru"]) {
               self.manager.lang = NTESVerifyCodeLangRUS;
           } else if ([language_type isEqualToString:@"ar"]) {
               self.manager.lang = NTESVerifyCodeLangKSA;
           } else if ([language_type isEqualToString:@"de"]) {
               self.manager.lang = NTESVerifyCodeLangDE;
           } else if ([language_type isEqualToString:@"it"]) {
               self.manager.lang = NTESVerifyCodeLangIT;
           } else if ([language_type isEqualToString:@"he"]) {
               self.manager.lang = NTESVerifyCodeLangHE;
           } else if ([language_type isEqualToString:@"hi"]) {
               self.manager.lang = NTESVerifyCodeLangHI;
           } else if ([language_type isEqualToString:@"id"]) {
               self.manager.lang = NTESVerifyCodeLangID;
           } else if ([language_type isEqualToString:@"my"]) {
               self.manager.lang = NTESVerifyCodeLangMY;
           } else if ([language_type isEqualToString:@"lo"]) {
               self.manager.lang = NTESVerifyCodeLangLO;
           } else if ([language_type isEqualToString:@"ms"]) {
               self.manager.lang = NTESVerifyCodeLangMS;
           } else if ([language_type isEqualToString:@"pl"]) {
               self.manager.lang = NTESVerifyCodeLangPL;
           } else if ([language_type isEqualToString:@"pt"]) {
               self.manager.lang = NTESVerifyCodeLangPT;
           } else if ([language_type isEqualToString:@"es"]) {
               self.manager.lang = NTESVerifyCodeLangES;
           } else if ([language_type isEqualToString:@"tr"]) {
               self.manager.lang = NTESVerifyCodeLangTR;
           } else {
               self.manager.lang = NTESVerifyCodeLangCN;
           }
       }
   }

    // 设置颜色
    self.manager.color = [UIColor blackColor];

    // 设置frame
    self.manager.frame = CGRectNull;
}

- (void)showCaptcha:(FlutterMethodCall*) call result:(FlutterResult)resultDict {
    [self.manager openVerifyCodeView:nil];
}

- (void)destroyCaptcha:(FlutterMethodCall*) call result:(FlutterResult)resultDict {
    [self.manager closeVerifyCodeView];
}

/*
* 验证码组件初始化完成
*/
- (void)verifyCodeInitFinish {
    NSMutableDictionary *dict = [NSMutableDictionary dictionary];
    [dict setValue:@"" forKey:@"code"];
    [dict setValue:@"验证码组件初始化完成" forKey:@"msg"];
    [dict setValue:@"" forKey:@"validate"];
    if ([CaptchaPluginFlutterPlugin sharedInstance].eventSink) {
         [CaptchaPluginFlutterPlugin sharedInstance].eventSink(dict);
    }
}

/**
* 验证码组件初始化出错
*
* @param message 错误信息
*/
- (void)verifyCodeInitFailed:(NSArray *)message {
   if (message.count != 0) {
       NSString *jsonString = [message objectAtIndex:0];
       NSData *jsonData = [jsonString dataUsingEncoding:NSUTF8StringEncoding];
       NSError *error=nil;
       NSDictionary *parsedData = [NSJSONSerialization JSONObjectWithData:jsonData options:kNilOptions error:&error];
       NSString *code = [parsedData objectForKey:@"code"];
       NSMutableDictionary *dict = [NSMutableDictionary dictionary];
       [dict setValue:code forKey:@"code"];
       [dict setValue:@"" forKey:@"msg"];
       [dict setValue:@"" forKey:@"validate"];
       if ([CaptchaPluginFlutterPlugin sharedInstance].eventSink) {
           [CaptchaPluginFlutterPlugin sharedInstance].eventSink(dict);
       }
   }
}

/**
* 完成验证之后的回调
*
* @param result 验证结果 BOOL:YES/NO
* @param validate 二次校验数据，如果验证结果为false，validate返回空
* @param message 结果描述信息
*
*/
- (void)verifyCodeValidateFinish:(BOOL)result validate:(NSString *)validate message:(NSString *)message{
   NSMutableDictionary *dict = [NSMutableDictionary dictionary];
   [dict setValue:@"" forKey:@"code"];
   [dict setValue:message forKey:@"msg"];
   [dict setValue:validate forKey:@"validate"];
   if ([CaptchaPluginFlutterPlugin sharedInstance].eventSink) {
        [CaptchaPluginFlutterPlugin sharedInstance].eventSink(dict);
   }
}

/**
* 关闭验证码窗口后的回调
*/
- (void)verifyCodeCloseWindow:(NTESVerifyCodeClose)close {
    NSMutableDictionary *dict = [NSMutableDictionary dictionary];
    [dict setValue:@"" forKey:@"code"];
    [dict setValue:@"关闭验证码窗口后的回调" forKey:@"msg"];
    [dict setValue:@"" forKey:@"validate"];
    if ([CaptchaPluginFlutterPlugin sharedInstance].eventSink) {
         [CaptchaPluginFlutterPlugin sharedInstance].eventSink(dict);
    }
}

/**
* 网络错误
*
* @param error 网络错误信息
*/
- (void)verifyCodeNetError:(NSError *)error {
    NSMutableDictionary *dict = [NSMutableDictionary dictionary];
    [dict setValue:@(error.code) forKey:@"code"];
    [dict setValue:[error localizedDescription] forKey:@"msg"];
    [dict setValue:@"" forKey:@"validate"];
    if ([CaptchaPluginFlutterPlugin sharedInstance].eventSink) {
        [CaptchaPluginFlutterPlugin sharedInstance].eventSink(dict);
    }
//   self.keepAliveCallback(dict, YES);
   NSLog(@"收到网络错误的回调:%@(%ld)", [error localizedDescription], (long)error.code);
}


@end
